@@ -8,8 +8,7 @@ git diff --quiet HEAD pyproject.toml
 rm -rf dist
 hatch version "$1"
 hatch build
-git add */__init__.py
+git add mkdocs_literate_nav/__init__.py
 git commit -m "v$1"
 git tag -a -m "" "v$1"
-hatch publish
 git push origin master --tags
