@@ -7,7 +7,8 @@ import logging
 import posixpath
 import urllib.parse
 import xml.etree.ElementTree as etree
-from typing import Any, Callable, Dict, Iterator, List, Optional, Tuple, Union, cast
+from collections.abc import Iterator
+from typing import TYPE_CHECKING, Callable, Optional, Union, cast
 
 import markdown
 import markdown.extensions
@@ -18,25 +19,58 @@ import mkdocs.utils
 
 from mkdocs_literate_nav import exceptions
 
+if TYPE_CHECKING:
+    from .plugin import MkDocsGlobber
+
+
 log = logging.getLogger(f"mkdocs.plugins.{__name__}")
 
 _unescape: Callable[[str], str]
 try:
-    _unescape = markdown.treeprocessors.UnescapeTreeprocessor().unescape  # type: ignore
+    _unescape = markdown.treeprocessors.UnescapeTreeprocessor().unescape
 except AttributeError:
-    _unescape = markdown.postprocessors.UnescapePostprocessor().run
+    _unescape = markdown.postprocessors.UnescapePostprocessor().run  # type: ignore[attr-defined]
 
 
-NavItem = Union[str, Dict[Optional[str], Union[str, Any]]]
-Nav = List[NavItem]
-RootStack = Tuple[str, ...]
+class Wildcard:
+    trim_slash = False
+
+    def __init__(self, *path_parts: str, fallback: bool = True):
+        norm = posixpath.normpath(posixpath.join(*path_parts).lstrip("/"))
+        if path_parts[-1].endswith("/") and not self.trim_slash:
+            norm += "/"
+        self.value = norm
+        self.fallback = path_parts[-1] if fallback else None
+
+    def __repr__(self):
+        return f"{type(self).__name__}({self.value!r})"
+
+
+if TYPE_CHECKING:
+    NavWithWildcardsItem = Union[
+        Wildcard,
+        str,
+        "NavWithWildcards",
+        dict[Optional[str], Union[Wildcard, str, "NavWithWildcards"]],
+    ]
+    NavWithWildcards = list[NavWithWildcardsItem]
+
+    NavItem = Union[str, dict[Optional[str], Union[str, "Nav"]]]
+    Nav = list[NavItem]
+
+    RootStack = tuple[str, ...]
+
+
+class DirectoryWildcard(Wildcard):
+    trim_slash = True
 
 
 class NavParser:
     def __init__(
         self,
         get_nav_for_dir: Callable[[str], tuple[str, str] | None],
-        globber,
+        globber: MkDocsGlobber,
+        *,
         implicit_index: bool = False,
         markdown_config: dict | None = None,
     ):
@@ -49,34 +83,32 @@ class NavParser:
 
     def markdown_to_nav(self, roots: tuple[str, ...] = (".",)) -> Nav:
         root = roots[0]
-        ext = _MarkdownExtension()
-        dir_nav = self.get_nav_for_dir(root)
-        if dir_nav:
-            nav_file_name, md = dir_nav
-            markdown_config = dict(
-                self._markdown_config,
-                extensions=[ext, *(self._markdown_config.get("extensions") or ())],
-            )
-            markdown.markdown(md, **markdown_config)
-            if ext.nav is not None:
+
+        if dir_nav := self.get_nav_for_dir(root):
+            nav_file_name, markdown_content = dir_nav
+            nav = _extract_nav_from_content(self._markdown_config, markdown_content)
+
+            if nav is not None:
                 self_path = posixpath.normpath(posixpath.join(root, nav_file_name))
                 if not (self.implicit_index and self_path == self.globber.find_index(root)):
                     self.seen_items.add(self_path)
-        first_item = None
-        if ext.nav is not None and self.implicit_index:
-            first_item = self.globber.find_index(root)
-            if first_item:
-                first_item = Wildcard(root, "/" + first_item, fallback=False)
-        if ext.nav is None:
-            log.debug(f"Navigation for {root!r} will be inferred.")
-            return self._resolve_wildcards([Wildcard(root, "*", fallback=False)], roots)
-        return self._resolve_wildcards(self._list_element_to_nav(ext.nav, root, first_item), roots)
+
+                first_item: Wildcard | None = None
+                if self.implicit_index:
+                    if found_index := self.globber.find_index(root):
+                        first_item = Wildcard(root, "/" + found_index, fallback=False)
+                return self._resolve_wildcards(
+                    self._list_element_to_nav(nav, root, first_item), roots
+                )
+
+        log.debug(f"Navigation for {root!r} will be inferred.")
+        return self._resolve_wildcards([Wildcard(root, "*", fallback=False)], roots)
 
     def _list_element_to_nav(
         self, section: etree.Element, root: str, first_item: Wildcard | str | None = None
-    ):
+    ) -> NavWithWildcards:
         assert section.tag in _LIST_TAGS
-        result: list[Wildcard | str | list | dict[str, Wildcard | str | list]] = []
+        result: NavWithWildcards = []
         if first_item is not None:
             if isinstance(first_item, str):
                 self.seen_items.add(first_item)
@@ -90,14 +122,13 @@ class NavParser:
             try:
                 child = next(children)
                 if not out_title and child.tag == "a":
-                    link = child.get("href")
-                    if link:
+                    if link := child.get("href"):
                         out_item = self._resolve_string_item(root, link)
                         out_title = _unescape("".join(child.itertext()))
                     child = next(children)
                 if child.tag in _LIST_TAGS:
                     out_item = self._list_element_to_nav(
-                        child, root, cast(Union[Wildcard, str, None], out_item)
+                        child, root, cast("Union[Wildcard, str, None]", out_item)
                     )
                     child = next(children)
             except StopIteration:
@@ -133,10 +164,10 @@ class NavParser:
             return DirectoryWildcard(root, link)
         return abs_link
 
-    def _resolve_wildcards(self, nav, roots: RootStack = (".",)) -> Nav:
-        def can_recurse(new_root):
+    def _resolve_wildcards(self, nav: NavWithWildcards, roots: RootStack = (".",)) -> Nav:
+        def can_recurse(new_root: str) -> bool:
             if new_root in roots:
-                rec = " -> ".join(repr(r) for r in reversed((new_root,) + roots))
+                rec = " -> ".join(repr(r) for r in reversed((new_root, *roots)))
                 self._warn(f"Disallowing recursion {rec}")
                 return False
             return True
@@ -154,21 +185,24 @@ class NavParser:
         for entry in nav:
             if isinstance(entry, dict) and len(entry) == 1:
                 [(key, val)] = entry.items()
+                new_val: str | Nav | None = None
                 if isinstance(val, list):
-                    entry[key] = self._resolve_wildcards(val, roots)
+                    new_val = self._resolve_wildcards(val, roots)
                 elif isinstance(val, DirectoryWildcard):
-                    entry[key] = (
-                        self.markdown_to_nav((val.value,) + roots)
+                    new_val = (
+                        self.markdown_to_nav((val.value, *roots))
                         if can_recurse(val.value)
                         else val.fallback
                     )
                 elif isinstance(val, Wildcard):
-                    entry[key] = self._resolve_wildcards([val], roots) or val.fallback
-                if entry[key]:
-                    resolved.append(entry)
+                    new_val = self._resolve_wildcards([val], roots) or val.fallback
+                else:
+                    new_val = val
+                if new_val:
+                    resolved.append({key: new_val})
                 continue
 
-            assert not isinstance(entry, (DirectoryWildcard, list))
+            assert not isinstance(entry, (DirectoryWildcard, list, dict))
             if not isinstance(entry, Wildcard):
                 resolved.append(entry)
                 continue
@@ -180,8 +214,7 @@ class NavParser:
                     continue
                 if self.globber.isdir(item):
                     title = mkdocs.utils.dirname_to_title(posixpath.basename(item))
-                    subitems = self.markdown_to_nav((item,) + roots)
-                    if subitems:
+                    if subitems := self.markdown_to_nav((item, *roots)):
                         resolved.append({title: subitems})
                 else:
                     if entry.value.endswith("/"):
@@ -197,68 +230,69 @@ class NavParser:
             return nav
         return self._resolve_wildcards([self._resolve_yaml_nav(x) for x in nav])
 
-    def _resolve_yaml_nav(self, item: NavItem):
+    def _resolve_yaml_nav(self, item) -> NavWithWildcardsItem:
         if isinstance(item, str) and "*" in item:
             return Wildcard("", item)
-        if isinstance(item, dict) and len(item) == 1:
+        if isinstance(item, dict):
+            assert len(item) == 1
             [(key, val)] = item.items()
             if isinstance(val, list):
-                val = [self._resolve_yaml_nav(x) for x in val]
-            elif isinstance(val, str) and "*" in val:
-                val = Wildcard("", val)
-            elif isinstance(val, str):
-                val = self._resolve_string_item("", val)
+                return {key: [self._resolve_yaml_nav(x) for x in val]}
+            if isinstance(val, str):
+                if "*" in val:
+                    return {key: Wildcard("", val)}
+                return {key: self._resolve_string_item("", val)}
             return {key: val}
         return item
 
 
-_NAME = "mkdocs_literate_nav"
-
-
-class _MarkdownExtension(markdown.extensions.Extension):
-    _treeprocessor: _Treeprocessor
-
-    @property
-    def nav(self) -> etree.Element | None:
-        try:
-            return self._treeprocessor.nav
-        except AttributeError:
-            return None
-
-    def extendMarkdown(self, md):
-        md.inlinePatterns.deregister("html", strict=False)
-        md.inlinePatterns.deregister("entity", strict=False)
-        md.preprocessors.register(_Preprocessor(md), _NAME, 25)
-        self._treeprocessor = _Treeprocessor(md)
-        md.treeprocessors.register(self._treeprocessor, _NAME, 19)
+def _extract_nav_from_content(markdown_config: dict, markdown_content: str) -> etree.Element | None:
+    md = markdown.Markdown(**markdown_config)
+    md.inlinePatterns.deregister("html", strict=False)
+    md.inlinePatterns.deregister("entity", strict=False)
+    preprocessor = _Preprocessor(md)
+    preprocessor._register()
+    treeprocessor = _Treeprocessor(md)
+    treeprocessor._register()
+    md.convert(markdown_content)
+    return treeprocessor.nav
 
 
 class _Preprocessor(markdown.preprocessors.Preprocessor):
-    def run(self, lines):
-        for line in lines:
+    nav_placeholder: str | None = None
+
+    def run(self, lines: list[str]) -> list[str]:
+        for i, line in enumerate(lines):
             if line.strip() == "<!--nav-->":
                 self.nav_placeholder = self.md.htmlStash.store("")
-                line = self.nav_placeholder + "\n"
-            yield line
+                lines[i] = self.nav_placeholder + "\n"
+        return lines
+
+    def _register(self) -> None:
+        self.md.preprocessors.register(self, "mkdocs_literate_nav", priority=25)
 
 
 class _Treeprocessor(markdown.treeprocessors.Treeprocessor):
-    nav: etree.Element
+    nav: etree.Element | None = None
 
-    def run(self, doc):
-        try:
-            nav_placeholder = self.md.preprocessors[_NAME].nav_placeholder
-        except AttributeError:
-            # Will look for the last list.
-            items = reversed(doc)
-        else:
+    def run(self, root: etree.Element) -> None:
+        preprocessor: _Preprocessor = self.md.preprocessors["mkdocs_literate_nav"]  # type: ignore[assignment]
+        nav_placeholder = preprocessor.nav_placeholder
+        items: Iterator[etree.Element]
+        if nav_placeholder is not None:
             # Will look for the first list after the last <!--nav-->.
-            items = itertools.dropwhile(lambda el: el.text != nav_placeholder, doc)
+            items = itertools.dropwhile(lambda el: el.text != nav_placeholder, root)
+        else:
+            # Will look for the last list.
+            items = reversed(root)
         for el in items:
             if el.tag in _LIST_TAGS:
                 self.nav = copy.deepcopy(el)
                 break
 
+    def _register(self) -> None:
+        self.md.treeprocessors.register(self, "mkdocs_literate_nav", priority=19)
+
 
 _LIST_TAGS = ("ul", "ol")
 _EXAMPLES = """
@@ -270,24 +304,6 @@ Examples:
 """
 
 
-class Wildcard:
-    trim_slash = False
-
-    def __init__(self, *path_parts: str, fallback: bool = True):
-        norm = posixpath.normpath(posixpath.join(*path_parts).lstrip("/"))
-        if path_parts[-1].endswith("/") and not self.trim_slash:
-            norm += "/"
-        self.value = norm
-        self.fallback = path_parts[-1] if fallback else None
-
-    def __repr__(self):
-        return f"{type(self).__name__}({self.value!r})"
-
-
-class DirectoryWildcard(Wildcard):
-    trim_slash = True
-
-
 def _iter_children_without_tail(element: etree.Element) -> Iterator[etree.Element]:
     for child in element:
         yield child
@@ -301,7 +317,7 @@ def _iter_children_without_tail(element: etree.Element) -> Iterator[etree.Elemen
 def _to_short_string(el: etree.Element) -> str:
     el = copy.deepcopy(el)
     for child in el:
-        if child:
+        if len(child):
             del child[:]
             child.text = "[...]"
     el.tail = None
